@@ -16,6 +16,11 @@ insert_brs <- function(vec) {
     ret
 }
 
+
+div_helper <- function(lst, class) {
+    do.call(tags$div, c(list(class = paste(class, "rtables-container"), lst)))
+}
+
 #' Convert an `rtable` object to a `shiny.tag` html object
 #'
 #' The returned `html` object can be immediately used in shiny and rmarkdown.
@@ -26,7 +31,6 @@ insert_brs <- function(vec) {
 #' @param class_td class for td tag
 #' @param class_th class for th tag
 #' @param width width
-#' @param caption_txt Caption text (not including label anchor) for the table
 #' @param link_label link anchor label (not including \code{tab:} prefix) for the table.
 #'
 #' @return A \code{shiny.tag} object representing \code{x} in HTML.
@@ -58,30 +62,29 @@ as_html <- function(x,
                     class_tr = "",
                     class_td = "",
                     class_th = "",
-                    caption_txt = NULL,
                     link_label = NULL) {
-  
+
   if (is.null(x)) {
     return(tags$p("Empty Table"))
   }
-  
+
   stopifnot(is(x, "VTableTree"))
-  
+
   mat <- matrix_form(x)
-  
+
   nrh <- attr(mat, "nrow_header")
   nc <- ncol(x) + 1
-  
+
   cells <- matrix(rep(list(list()), (nrh + nrow(x)) * (ncol(x) + 1)),
                   ncol = ncol(x) + 1)
-  
+
   for(i in unique(mat$line_grouping)) {
     rows <- which(mat$line_grouping == i)
     for(j in 1:ncol(mat$strings)) {
       curstrs <- mat$strings[rows,j]
       curspans <- mat$spans[rows,j]
       curaligns <- mat$aligns[rows,j]
-      
+
       curspn <- unique(curspans)
       stopifnot(length(curspn) == 1)
       inhdr <- i <= attr(mat, "nrow_header")
@@ -92,7 +95,7 @@ as_html <- function(x,
       cells[i, j][[1]] <- do.call(tagfun, c(insert_brs(curstrs), args))
     }
   }
-  
+
   ## special casing hax for top_left. We probably want to do this better someday
   cells[1:nrh, 1] <- mapply(
     FUN = function(x, algn) {
@@ -102,7 +105,7 @@ as_html <- function(x,
     algn = mat$aligns[1:nrh, 1],
     SIMPLIFY = FALSE
   )
-  
+
   # indent row names
   for (i in seq_len(nrow(x))) {
     indent <- mat$row_info$indent[i]
@@ -111,23 +114,44 @@ as_html <- function(x,
                                                                style = paste0("padding-left: ", indent * 3, "ch"))
     }
   }
-  
+
   cells[!mat$display] <- NA_integer_
-  
+
   rows <- apply(cells, 1, function(row) {
     do.call(tags$tr, c(Filter(function(x) !identical(x, NA_integer_), row), list(class = class_tr)))
   })
-  
-  if(!is.null(caption_txt)) {
-    if(!is.null(link_label))
-      labtxt <- sprintf("(#tab:%s)", link_label)
-    else
-      labtxt <- NULL
-    captxt <- paste(labtxt, caption_txt)
-    captag <- tags$caption(class = "Table Caption", captxt)
-  } else {
-    captag <- NULL
-  }
-  do.call(tags$table, c(rows, list(class = class_table), if(!is.null(captag)) list(captag)))
-  
+
+    hdrtag <- div_helper( class = "rtables-titles-block",
+                         list(div_helper(class = "rtables-main-titles-block",
+                                         lapply(main_title(x), tags$p, class="rtables-main-title")),
+                              div_helper(class = "rtables-subtitles-block",
+                                         lapply(subtitles(x), tags$p, class="rtables-subtitle"))))
+
+
+    tabletag <- do.call(tags$table, c(rows, list(class = class_table, tags$caption(sprintf("(\\#tag:%s)", link_label), style="caption-side:top;", .noWS = "after-begin", hdrtag))))
+
+    rfnotes <- div_helper(class = "rtables-ref-footnotes-block",
+                                    lapply(mat$ref_footnotes, tags$p, class="rtables-referential-footnote"))
+
+    mftr <- div_helper(class = "rtables-main-footers-block",
+                       lapply(main_footer(x), tags$p, class = "rtables-main-footer"))
+
+    pftr <- div_helper(class = "rtables-prov-footers-block",
+                       lapply(prov_footer(x), tags$p, class="rtables-prov-footer"))
+
+    ## XXX this omits the divs entirely if they are empty. Do we want that or do we want them
+    ## to be there but empty??
+    ftrlst <- list(if(length(mat$ref_footnotes) > 0) rfnotes,
+                   if(length(main_footer(x)) > 0) mftr,
+                   if(length(prov_footer(x)) > 0) pftr)
+
+    ftrlst <- ftrlst[!vapply(ftrlst, is.null, TRUE)]
+
+    ftrtag <- div_helper(class = "rtables-footers-block",
+                         ftrlst)
+
+    div_helper(class = "rtables-all-parts-block",
+               list(#hdrtag,
+                    tabletag,
+                    ftrtag))
 }
