@@ -163,31 +163,6 @@ visible_only and not")
 
 
 
-    ## test_rowpaths(tab)
-    ## pdf <- make_row_df(tab)
-    ## ##res <- lapply(pdf$path, function(pth) cell_values(tab, pth))
-    ## ## we expect true here cause we're checking for errors rather than against values
-    ## test_rowpaths(tab)
-    ## expect(TRUE, "some paths in visible_only pag_df did not work")
-    ## ##pdf2 <- make_row_df(tab, visible_only = FALSE)
-
-    ## ##res2 <- lapply(pdf2$path, function(pth) cell_values(tab, pth))
-    ## test_rowpaths(tab, FALSE)
-    ## expect(TRUE, "some paths in full structure pag_df did not work")
-
-    ## ## subset_cols <- rtables:::subset_cols
-    ## ## cdf <- make_col_df(tab)
-    ## ## res3 <- lapply(cdf$path, function(pth) subset_cols(tab, pth))
-    ## test_colpaths(tab)
-    ## expect(TRUE, "some paths in visible_only make_col_df did not work")
-    ## cdf2 <- make_col_df(tab, visible_only=FALSE)
-    ## cdf2sub <- cdf2[-c(1,4),]
-    ## row.names(cdf2sub) <- NULL
-    ## expect_identical(cdf, cdf2sub)
-    ## res4 <- lapply(cdf2$path, function(pth) subset_cols(tab, pth))
-    ## expect(TRUE, "some paths in visible_only=FALSE make_col_df did not work")
-    ## ## root omitted
-    ## expect_identical(cdf2$name[1], "ARM1")
 
     combodf <- tibble::tribble(
                            ~valname, ~label, ~levelcombo, ~exargs,
@@ -272,3 +247,90 @@ test_that("setters work ok", {
 })
 
 
+test_that("cell_values and value_at work on row objects", {
+
+    tbl <- basic_table() %>%
+        split_cols_by("ARM") %>%
+        split_cols_by("STRATA2") %>%
+        analyze("AEDECOD") %>%
+        build_table(ex_adae, ex_adsl)
+
+    first_row <- collect_leaves(tbl)[[1]]
+
+    va <- value_at(first_row, colpath = c("ARM", "A: Drug X", "STRATA2", "S2"))
+
+    cv <- cell_values(first_row, colpath = c("ARM", "C: Combination"))
+
+    expect_identical(va, 33L)
+
+    expect_identical(cv,
+                     setNames(list(32L, 56L),
+                              c("C: Combination.S1",
+                                "C: Combination.S2")))
+})
+
+test_that("label_at_path works", {
+    lyt <- make_big_lyt()
+
+    tab <- build_table(lyt, rawdat)
+    orig_labs <- row.names(tab)
+
+    tab4 <- tab
+
+    label_at_path(tab4, c("root", "RACE", "WHITE", "FACTOR2", "B", "AGE")) <- NA_character_
+
+    expect_identical(row.names(tab4), orig_labs[-9])
+
+    tab5 <- tab
+
+    newlab5 <- "race var label"
+    label_at_path(tab5, c("root", "RACE")) <-  newlab5
+    expect_identical(row.names(tab5), c(newlab5, orig_labs))
+
+    rps <- row_paths(tab)
+
+    labs <- vapply(rps, function(pth) label_at_path(tab, pth), "", USE.NAMES=FALSE)
+    expect_identical(labs, orig_labs)
+
+    newthangalangs <- paste(orig_labs, "redux")
+
+    tab7 <- tab
+    for(i in seq_along(orig_labs))
+        label_at_path(tab7, rps[[i]]) <- newthangalangs[i]
+
+    expect_identical(newthangalangs,
+                     row.names(tab7))
+})
+
+test_that("insert_row_at_path works", {
+ lyt <- basic_table() %>%
+     split_rows_by("COUNTRY", split_fun = keep_split_levels(c("CHN", "USA"))) %>%
+     summarize_row_groups() %>%
+   analyze("AGE")
+
+ tab <- build_table(lyt, DM)
+ orig_rns <- row.names(tab)
+ tab2 <- insert_row_at_path(tab, c("COUNTRY", "CHN", "AGE", "Mean"),
+                           rrow("new row", 555))
+ expect_identical(row.names(tab2),
+                  c(orig_rns[1],
+                    "new row",
+                    orig_rns[-1]))
+
+ tab3 <- insert_row_at_path(tab2, c("COUNTRY", "CHN", "AGE", "Mean"),
+                           rrow("new row redux", 888),
+                           after = TRUE)
+ expect_identical(row.names(tab3),
+                  c(orig_rns[1],
+                    "new row",
+                    orig_rns[2],
+                    "new row redux",
+                    orig_rns[-c(1:2)]))
+
+ myrow <- rrow("whaaat", 578)
+ rps <- row_paths(tab)
+ expect_error(insert_row_at_path(tab, c("root", "COUNTRY"), myrow))
+ expect_error(insert_row_at_path(tab, c("root", "COUNTRY", "CHN"), myrow))
+ expect_error(insert_row_at_path(tab, c("root", "COUNTRY", "CHN", "AGE"), myrow))
+ expect_error(insert_row_at_path(tab, rps[[1]], myrow))
+})
